/**
 * @defgroup config_app_c config_app.c
 * @ingroup Synergy_Template
 *
 * @brief Configure application-wide settings
 * 
 * Configure the application parameters here.  If you have permanent storage, like a USB
 * memory or access to the flash file system, load the configuration from there.
 *
 * @{
 */

#include "cogentcetk.h"
#include "config_app.h"

// To pick up the project configuration for MAC address.  You can
// delete this if you are using your own method to configure MAC
// address.
#include "sf_el_nx_cfg.h"

/** 
 * Initialize the application configuration object.  This object must
 * be allocated before entry to this function, and is typically
 * created as a global static variable in the @ref mainline_c
 * mainline.  You can modify this structure to suit your application
 * requirements.
 * 
 * In the example code the contents of this object are hard-coded.  In
 * a production system you may prefer to store this information in
 * persistent storage and then read it into the CAppConfig object
 * here.
 * 
 * @param config 
 */
void APP_InitializeConfig(CAppConfig* config)
{
	config->useSsl = 0;
	config->useWebsocket = 1;
	config->portname = "80";
	config->domain = "demo_ETK_Modbus";
	config->username = "demo/guest";
	config->password = "guest";
	config->pwtype = PASSWORD_PLAIN;
	config->pollUsecs = 10000;
	config->retrySecs = 15;
	config->disconnectSecs = 0;

	config->heartbeat = UT_SecsToClock(10, 0);
	config->timeout = UT_SecsToClock(30, 0);
	config->retry = UT_SecsToClock(config->retrySecs, 0);

	config->heapBytes = APP_HeapSize;

	// If we have a DNS server then we can use host names
#if INIT_DNS
	config->hostname = "demo.skkynet.com";
#else
	config->hostname = "192.168.4.90";
#endif

	// If we have a DNS name for our Modbus slave, we could provide it here.
	config->modbusHost = "192.168.4.90";
	config->modbusPort = "502";
	config->modbusPollMs = 10;
	config->modbusSlaveId = 1;

	// We need to set the MAC address somehow.  You will probably need
	// to read it from ROM or flash.  Here we are setting it using a
	// hard-coded value from the project configuration, or setting it
	// to a hard-coded value if no project value is set.
#ifdef SF_EL_NX_CFG_ENET1_MAC_H
	config->macAddressLsw = SF_EL_NX_CFG_ENET1_MAC_L;
	config->macAddressMsw = SF_EL_NX_CFG_ENET1_MAC_H;
#else
	config->macAddressLsw = 0x00c06e47;
	config->macAddressMsw = 0x0000feed;
#endif
}

CDataHubPoint* APP_CreateAndSetPoint(CTCPConnectionContainer *cc, CTCPClient* client, char* name, int flags, INT64 value)
{
	CDataHubPoint			*point;
	point = CTCPConnectionContainer_CreatePoint(cc, name, flags);
	APP_SetPointInt(point, value);

	/* Explicitly register for data events for any points we intend to
	   get from the DataHub.  We are only acting as an outgoing source for
	   most points, but some we need back. */
	if (flags & PT_FLAG_WRITABLE)
		CTCPClient_RegisterPoint(client, name);
	return point;
}

void APP_SetPointInt(CDataHubPoint *point, INT64 value)
{
	CDataHubPoint_setQuality(point, PT_QUALITY_GOOD);
	CDataHubPoint_setValueInt(point, value);
	CDataHubPoint_setTimeStampDATE(point, UT_GetCurrentPointTime());
}

#if ! NO_FLOATING_POINT
void APP_SetPointDouble(CDataHubPoint *point, double value)
{
	CDataHubPoint_setQuality(point, PT_QUALITY_GOOD);
	CDataHubPoint_setValueDouble(point, value);
	CDataHubPoint_setTimeStampDATE(point, UT_GetCurrentPointTime());
}
#endif

void APP_SetPointString(CDataHubPoint *point, char* value)
{
	CDataHubPoint_setQuality(point, PT_QUALITY_GOOD);
	CDataHubPoint_setValueString(point, value);
	CDataHubPoint_setTimeStampDATE(point, UT_GetCurrentPointTime());
}

/**
 * @}
 */
