/**
 * @defgroup   config_app_h config_app.h
 * @ingroup Synergy_Template
 * 
 * @brief  Define application-specific configuration
 * 
 * This file contains a definition of the CAppConfig structure that is
 * passed to the application-specific configuration functions during
 * application start-up.  You can modify the CAppConfig strucure to
 * suit your application requirements.
 *
 * @{
 */

#if defined(BSP_BOARD_S7G2_SK)
#define SYNERGY_ETHERNET_DRIVER			nx_ether_driver_eth1
#endif

#if defined(BSP_BOARD_S7G2_DK)
#define SYNERGY_ETHERNET_DRIVER			nx_ether_driver_eth1
#endif

/**
 * @ingroup ThreadxMemory
 * 
 * The APP_HeapSize is used to determine the amount of memory to
 * reserve for user thread stacks, internal structures, inter-thread
 * messages and #CDataHubPoint structures.  You application will consume
 * memory from this heap depending on the number of connections,
 * threads, points and timers that are configured.  During development
 * you can look at the performance of the heap by examining the global
 * variables #ME_Total, #ME_Hiwater, #ME_Nallocs, #ME_Nfrees,
 * #ME_Nreallocs and #ME_Hiaddress.
 *
 * Each user thread requires a stack whose size is defined by
 * #ETK_THREAD_STACK_SIZE.
 *
 * Each data point requires approximately 500 bytes.  This varies with
 * the length of the point name.
 *
 * Each #CTimer requires approximately 80 bytes.
 *
 * Each inter-thread message in flight requires 56 bytes, plus the
 * length of a string if the message contains a point change
 * notification for which the value is a string type.
 * 
 * We need to configure the heap size with a constant because we need
 * to reserve heap early in the initialization process, before we have
 * set the application configuration.  This means that we cannot
 * configure the application heap size dynamically from a
 * configuration file.
 *
 * With Renesas Synergy, you can override the default heap size in the
 * properties list of the Skkynet ETK module.
 */
#ifdef ETK_USER_HEAP_SIZE
#define APP_HeapSize	ETK_USER_HEAP_SIZE
#else
#define APP_HeapSize	65536
#endif

/**
 * The number of bytes in the Skkynet ETK memory heap.  See @ref
 * APP_HeapSize "APP_HeapSize" for more information.
 */
#define APP_HeapSize_ APP_HeapSize

/**
 * Defines an application-specific structure containing information
 * that is required during start-up.  In the example code, this
 * information is supplied in config_app.c.  In other instances this
 * information might be stored as persistent configuration in flash
 * memory.  An application developer should add or remove members in
 * this structure to suit the application requirements.
 */
typedef struct CAppConfig
{
	CTCPConnectionContainer *cc;			/**< Needed by many ETK API calls */
	CTCPClient				*client;		/**< The client that connects to the DataHub or SkkyHub server */
	int						useWebsocket;	/**< 0 or 1, indicating whether to connect using WebSocket */
	int						useSsl;			/**< 0 or 1, indicating whether to use SSL (not implemented) */
	char					*hostname;		/**< The name of the DataHub/SkkyHub server */
	char					*portname;		/**< The port number (as a string) to connect to on the DataHub/SkkyHub server */
	char					*domain;		/**< The data domain into which data will be stored */
	char					*username;		/**< The user name for authentication on the server */
	char					*password;		/**< The password for authentication on the server */
	int						pwtype;			/**< The type of password encoding */
	int						pollUsecs;		/**< The number of microseconds per poll.  ThreadX has a 10ms tick, so this should be a multiple of 10000 */
	int						retrySecs;		/**< The number of seconds between socket connection re-tries when connecting to the server*/
	int						disconnectSecs;	/**< The number of seconds to wait before disconnecting after a connection to the server is made.  This has the effect of periodically connecting and updating data, then disconnecting again. */
	MSCLOCK					heartbeat;		/**< The keep-alive heartbeat rate */
	MSCLOCK					timeout;		/**< The keep-alive timeout.  If no data or heartbeat is received from the server within this time, disconnect the socket. */
	MSCLOCK					retry;			/**< Internal.  Use #retrySecs instead. */

	// Memory limits
	int						heapBytes;		/**< Internal.  Holds the configured heap size. */

	// Modbus connection settings
	char					*modbusHost;	/**< The address of the Modbus slave device */
	char					*modbusPort;	/**< The port number (as a string) for the Modbus slave device */
	int						modbusSlaveId;	/**< The Modbus slave ID (1-254) */
	int						modbusPollMs;	/**< The number of milliseconds between Modbus polls */

	// Ethernet MAC address
	ULONG					macAddressLsw;	/**< The MAC address low 32 bits */
	ULONG					macAddressMsw;	/**< The MAC address high 16 bits. The high 16 bits of this value must be zero */
} CAppConfig;

/*
 * Define which components will be included in this application
 */

#ifdef ETK_USER_USE_DNS
#define INIT_DNS		ETK_USER_USE_DNS	/**< Allow the mainline to assign the DNS server */
#else
#define INIT_DNS		1	/**< Allow the mainline to assign the DNS server */
#endif

#ifdef ETK_USER_USE_DHCP
#define INIT_DHCP		ETK_USER_USE_DHCP	/**< Allow the mainline to assign the IP address using DHCP */
#else
#define INIT_DHCP		1	/**< Allow the mainline to assign the IP address using DHCP */
#endif

#define INIT_GATEWAY	1	/**< Allow the mainline to assign the IP gateway */

#define     STATIC_SERVER_IP_ADDRESS    IP_ADDRESS(192,168,0,2)  /**< The static IP address if DHCP is not enabled */
#define		STATIC_SERVER_NETMASK		0xFFFFFF00UL			 /**< The netmask if DHCP is not enabled */
#define     STATIC_DNS_SERVER_ADDRESS   IP_ADDRESS(8,8,8,8)		 /**< The DNS server address if DHCP is not enabled */
#define     STATIC_IP_GATEWAY_ADDRESS   IP_ADDRESS(192,168,0,1)  /**< The gateway address if DHCP is not enabled */

CDataHubPoint* APP_CreateAndSetPoint(CTCPConnectionContainer *cc, CTCPClient* client, char* name, int flags, INT64 value);
void APP_SetPointInt(CDataHubPoint *point, INT64 value);
void APP_SetPointDouble(CDataHubPoint *point, double value);
void APP_SetPointString(CDataHubPoint *point, char* value);

void APP_ConfigureTimers(CAppConfig* config);
void APP_ConfigurePoints(CAppConfig* config);
void APP_ConfigureModbus(CAppConfig *config);
void APP_ConfigureUserThreads(CAppConfig *config);
void APP_InitializeConfig(CAppConfig* config);

/**
 * @}
 */
